IDA Network Stack Repository
====

This repository contains the various projects that use the LwIP or IDA-LwIP Stack.
It includes a number of git submodules, so remember to always do a 'git submodule update' after switching branches

Getting started
----

Fork and clone the repository
```
git clone git@git.ida.ing.tu-bs.de:<YOUR_USER>/idaCom.git <YOUR_FOLDER>
git remote add upstream git@git.ida.ing.tu-bs.de:IDA_LWIP/idaCom.git
```

Check out the submodules
```
git submodule init
git submodule update
```

Whenever you need to modify a submodule, fork it and work on your fork. Example
```
cd src/Modules/lwip/
git remote -v
	origin	git@git.ida.ing.tu-bs.de:IDA_LWIP/ida-lwip.git (fetch)
	origin	git@git.ida.ing.tu-bs.de:IDA_LWIP/ida-lwip.git (push)
git remote add fork git@git.ida.ing.tu-bs.de:<YOUR_NAME>/ida-lwip.git
git remote -v
	origin	git@git.ida.ing.tu-bs.de:IDA_LWIP/ida-lwip.git (fetch)
	origin	git@git.ida.ing.tu-bs.de:IDA_LWIP/ida-lwip.git (push)
	fork	git@git.ida.ing.tu-bs.de:<YOUR_NAME>/ida-lwip.git (fetch)
	fork	git@git.ida.ing.tu-bs.de:<YOUR_NAME>/ida-lwip.git (push)
git checkout -b feature
git add... git commit...
git push fork feature
```

Repository Structure
----

You have to specify three different variables for build
APP: Your application/project. 
ARCH: Selected Architecture
CORE: Core Number

Example: make APP=ida_lwip_sllet_example ARCH=psur5 CORE=0 -j

TO PREVENT MERGE CONFLICTS:
Copy one of the example APPs and work in your own APP directory.
Copy one of the example Lauterbach Configs and work on your own.

ARCH examples:
	stm32f7: 	Cortex M7 on STM32F7 Disco Board		(deprecated)
	ps7:		Cortex A9 on Zynq 7000 					(ZC706 Board, deprecated)
	psua53:		Cortex A53 on Zynq Ultrascale+ 			(ZCU102 Board)
	psur5:		Cortex R5 on Zynq Ultrascale+ 			(ZCU102 Board)

Naming convention and Folder Structure has to be preserved to ensure Makefile is working

The folder structure is as follows:

```
<YOUR_FOLDER>
|--- Debug/
|    |--- $(ARCH)							/* Architecture dependent debug config templates */
|--- Makefile
|--- README.md
|--- src/
|    |--- APP/
|    |    |--- $(APP)/
|    |         |--- $(ARCH)/
|    |              |--- core$(CORE)/
|    |                   |--- build/					/*Build configuration used by makefile (sources.mk, includes.mk, config.mk)
|    |                   |    |--- sources.mk			/* List of source files */
|    |                   |    |--- includes.mk       		/* List of include paths */
|    |                   |    |--- config.mk         		/* Compiler/Linker flags */
|    |                   |--- cfg/
|    |                   |    |--- 	/* Typically used for various header files that are project specific */
|    |                   |--- linker/
|    |                   |    |--- 	/* Various Linker files */
|    |                   |--- src/
|    |                        |--- 	/* Various source files */
|    |--- Modules/							/* This is a folder for external modules */
|    |    |--- all the git submodules
|    |--- ucos_v1_5							/* ucos Sources by micrium */
|    |--- Xilinx							/* Xilinx BSP for various projects */
|--- out/ 								/* Generated compiler output, is removed during "make clean" */
```

Current Example Projects
----

dummy: Just a dummy for psur5, starting OS, doing nothing

ida_lwip_example: Example for use of ida_lwip, partly replaced by ida_lwip_sllet_example

ida_lwip_sllet_example: Showing split network stack with SL-LET and demo DDS middleware (very rough state, no real DDS, just for eval purpose, don't blame me for the spaghetti code)

lwip_example: Example of classic lwip on psua53 (running) and psur5 (not running at the moment due to memory restrictions)

lwip_sllet_example: Same as ida_lwip_sllet_example, except of filter-stack

Build Checker
----

Execute buildChecker.sh to see if all projects compile well


