/*
 * app.c
 *
 *  Created on: Aug 12, 2020
 *      Author: kaige
 */

#include <stdio.h>
#include "cpu_core.h"
#include "ucos_ii.h"
#include "ucos_bsp.h"

#include "netif/xemacpsif.h"
#include "netif/xadapter.h"
#include "lwip/init.h"
#include "lwip/inet.h"
#include "lwip/opt.h"
#include "lwip/ip_addr.h"


#include "lwipopts.h"
#include <ucos_int.h>
#include "lwip/sockets.h"

#include "xipipsu.h"
#include "xscugic.h"

#include "netif/xemacps_ieee1588.h"
#include "ida-lwip/ida_lwip_init.h"
#include "ida-lwip/ida_lwip_filter.h"

#include "dma/ida_lwip_dma_if.h"

//#include "xzdma_if.h"

#include "ILM/if/ilm_core.h"
#include "xil_mpu.h"

#include "imw_reader.h"
#include "imw_writer.h"
#include "imw_sync.h"

#include "ptpd.h"

#include "imw_example_configs.h"

extern struct netif EthDev;
IDA_LWIP_DMA_INSTANCE DMAInstance __attribute__ ((aligned (16))) __attribute__ ((section (".psu_lwip_tcm")));


#define TICKS_PER_S 			10000	//10kHz
#define TICKS_PER_MACRO_PERIOD 	100	//10Hz PWM, 20ms

extern const ILM_LET_TABLE_T CO_LET_TABLE[];
extern const ILM_HW_LET_TABLE_T C0_HW_Master_Table[];
extern const size_t CO_LET_TABLE_SIZE;
extern const size_t C0_HW_TABLE_SIZE;

static CPU_STK imwWriterTask1Stk[IMW_WRITER_TASK_1_STACK_SIZE];
static CPU_STK imwWriterTask3Stk[IMW_WRITER_TASK_3_STACK_SIZE];
static CPU_STK imwRxTask1Stk[IMW_RX_TASK_1_STACK_SIZE];

IMW_OBJECT_t imwObject1 = {
		.objectId=IMW_OBJECT_1_ID,
		.size=IMW_OBJECT_1_SIZE,
		.historyCount=4,
		.baseAddr=(void*)IMW_OBJECT_1_BASE
};

IMW_OBJECT_t imwObject2 = {
		.objectId=IMW_OBJECT_2_ID,
		.size=IMW_OBJECT_2_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_2_BASE
};

IMW_OBJECT_t imwObject3 = {
		.objectId=IMW_OBJECT_3_ID,
		.size=IMW_OBJECT_3_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_3_BASE
};

uint8_t txBuffer[1500] __attribute__ ((aligned (8)));
uint8_t txBuffer3[1500] __attribute__ ((aligned (8)));
uint8_t rxBuffer[1500] __attribute__ ((aligned (8)));

struct sockaddr_in board1 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,10),
		.sin_port = PP_HTONS(1000)
};

struct sockaddr_in board2 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,11),
		.sin_port = PP_HTONS(1000)
};

struct sockaddr_in board2_1 = {
		.sin_family = AF_INET,
		.sin_addr = IPADDR4_INIT_BYTES(192,168,178,11),
		.sin_port = PP_HTONS(1020)
};

IMW_READER_INTERNAL_t imwReader2Internal;

IMW_READER_CONFIG_t imwReader2Config = {
		.object = &imwObject2,
		.seqNrOffset = 0,
		.fragmentSize = 1440,
#if defined(IN_CHUNKS)
		.fragmentsPerChunk = 10,
#endif
		.padding = 4,
		.priority = 5,
		.maxRetry = 1,
		.target = (void*)&board2,
		.localPort = 1000,
		.rxBuffer = (void*)rxBuffer,
		.txBuffer = (void*)txBuffer,
		.internal = &imwReader2Internal
};

IMW_RX_TASK_INTERNAL_t imwRxTaskConfigInternal;

IMW_RX_TASK_CONFIG_t imwRxTaskConfig = {
		.internal = &imwRxTaskConfigInternal,
		.ready = 0,
		.taskStack = (void*)imwRxTask1Stk,
		.taskStackSize = IMW_RX_TASK_1_STACK_SIZE,
		.taskPrio = IMW_RX_TASK_1_PRIO,
		.rxBuffer = (void*)rxBuffer,
		.handleSubmessageData = imw_reader_handleSubmessageData,
		.handleSubmessageBitmap = imw_writer_handleSubmessageBitmap,
};

IMW_BITMAP_t writer1Bitmap;
IMW_WRITER_INTERNAL_t writer1ConfigInternal;

IMW_WRITER_CONFIG_t writer1Config = {
		.object=&imwObject1,
		.seqNrOffset = 3,
		.fragmentSize=1440,
#if defined(IN_CHUNKS)
		.fragmentsPerChunk = 10,
#else
		.fragmentsPerChunk = 57,
#endif
		.padding=4,
		.priority = 5,
		.target=(void*)&board2,
		.localPort = 2000,
		.bitmap = &writer1Bitmap,
		.txBuffer=(void*)&txBuffer,
		.txBufferSize = 1500,
		.rxBuffer=(void*)&rxBuffer,
		.rxBufferSize = 1500,
		.internal=&writer1ConfigInternal,
		.period = 50,
		.taskStack = (void*)imwWriterTask1Stk,
		.taskStackSize = IMW_WRITER_TASK_1_STACK_SIZE,
		.taskPrio = IMW_WRITER_TASK_1_PRIO,
		.taskId = IMW_WRITER_TASK_1_ID,
		.rxTaskConfig = &imwRxTaskConfig,
};

IMW_BITMAP_t writer3Bitmap;
IMW_WRITER_INTERNAL_t writer3ConfigInternal;

IMW_WRITER_CONFIG_t writer3Config = {
		.object=&imwObject3,
		.seqNrOffset = 3,
		.fragmentSize=1440,
#if defined(IN_CHUNKS)
		.fragmentsPerChunk = 10,
#else
		.fragmentsPerChunk = 57,
#endif
		.padding=4,
		.priority = 2,
		.target=(void*)&board2_1,
		.localPort = 2020,
		.bitmap = &writer3Bitmap,
		.txBuffer=(void*)&txBuffer3,
		.txBufferSize = 1500,
		.rxBuffer=(void*)&rxBuffer,
		.rxBufferSize = 1500,
		.internal=&writer3ConfigInternal,
		.period = 50,
		.taskStack = (void*)imwWriterTask3Stk,
		.taskStackSize = IMW_WRITER_TASK_3_STACK_SIZE,
		.taskPrio = IMW_WRITER_TASK_3_PRIO,
		.taskId = IMW_WRITER_TASK_3_ID,
		.rxTaskConfig = &imwRxTaskConfig,
};

void startAPP(void){
	Xil_SetMPURegion(IMW_OBJECT_1_BASE, 0x800000, NORM_SHARED_NCACHE | PRIV_RW_USER_RW);

	ILM_HW_Set_Interface((struct xemac_s *) EthDev.state);
	ILM_Init((ILM_LET_TABLE_T *) CO_LET_TABLE, CO_LET_TABLE_SIZE,TICKS_PER_MACRO_PERIOD);
	ILM_HW_Init((ILM_HW_LET_TABLE_T *) C0_HW_Master_Table, C0_HW_TABLE_SIZE,TICKS_PER_S, TICKS_PER_MACRO_PERIOD);

	ida_lwip_dma_init(&DMAInstance);

	imw_reader_Init();

	imw_sync_init();

	imw_net_rx_task_init(&imwRxTaskConfig);

	imw_writer_create(&writer1Config);
	imw_writer_create(&writer3Config);

	int sockfd = imw_reader_create(&imwReader2Config);
	imw_net_rx_register_handler(&imwRxTaskConfig,sockfd,(void*)&imwReader2Config);

	while (!ptpd_is_in_sync()) {
		OSTimeDly(OS_TICKS_PER_SEC/100);
	}

	OSTimeDly(10*OS_TICKS_PER_SEC);

	ip_addr_t ipaddrBoard2 = IPADDR4_INIT_BYTES(192,168,178,11);
	TimeInternal startTime;
	imw_sync(&ipaddrBoard2, 1, &startTime);

	ILM_HW_StartTimerAt(startTime.seconds);

//	ILM_HW_StartTimer();


	return;
}
