import sys
import os
#import time
import numpy as np

class imw_object:  
    def __init__(self, objId, startTime):  
        self.objId = objId  
        self.startTime = startTime
        self.deltaTime = {}
        self.deltaTime2 = {}
        
outfolder="./results/"

#INPUTS
filename=sys.argv[1]
basename=os.path.splitext(filename)[0]
if("board1" in basename):
	txRequired=4999
	rxRequired=0
	txFragments=57
	rxFragments=0
else:
	txRequired=0
	rxRequired=4999
	txFragments=0
	rxFragments=57

if("dma" in basename):
	dma = 1
else:
	dma = 0
	
chunkDelay = 1000000
if("shortdelay" in basename):
	chunkDelay = 500000

currentChunk = 0
currentSeq = 0

lastActHookTime = 0
timingCorrection = 0

# Result variables
imw_object_tx_list = {}
imw_object_rx_list = {}
imw_object_idle_list = {}
idle_time = 0
idle_cnt = 0
idle_check_time = 0

globalTs = {}
globalTs["lastIRQStartTime"] = 0
globalTs["lastCtxSwTime"] = 0


timing = { 	\
	"irqDurations" : [], \
	"rxIrqDurations": [], \
	"txIrqDurations": [], \
	"rxFilterResponseTime": [], \
	"rxFilterExecutionTime": [], \
	"txFilterResponseTime": [], \
	"txFilterExecutionTime": [], \
	"ctxSwTime": [], \
}

globalTime = 0
deltaTime = 0
app_start_time = 0
app_duration = 0
testRunning = 0

lastActivation=0
lineNr=0

def convert_value(value):
	val = {}
	val["value"] = value
	val["value_u32"] = value[0:8] if len(value) == 16 else "00000000"
	val["value_l32"] = value[8:16] if len(value) == 16 else value[0:8]
	val["value_u16"] = value[0:4]
	val["value_l16"] = value[4:8]
	return val

def parseLine(hookID, value, deltaTime, globalTime):
	global skb_open_list, skb_lifetime_list, tt, times, globalTs, udp_rcv_times
	global imw_object_tx_list, imw_object_rx_list, lastActivation, time_s, time_ns, lineNr, idle_cnt, app_start_time, idle_check_time
	global testRunning
	global dma
	global currentChunk, currentSeq
	global chunkDelay
	global lastActHookTime
	global timingCorrection
	val = convert_value(value)
	
	if(hookID == "0010"):
		globalTs["lastTimeTick"] = globalTime
	
	if(hookID == "0004"):
		if(testRunning == 0):
			testRunning = 1
		objseqNr = int(val["value_l16"],16)
		imw_object_tx_list[objseqNr] = imw_object(objseqNr,globalTime)
		if(objseqNr == 0):
			app_start_time = globalTime
			
			
	if(hookID == "000A"):
		objseqNr = int(val["value_u16"],16)
		fragId = int(val["value_l16"],16)
		try:
			obj = imw_object_tx_list[objseqNr]
			obj.deltaTime[fragId] = globalTime - obj.startTime
		except:
			print("Can not find tx obj with id "+val["value_u16"])
			pass

			
	if(hookID == "0005"):
		objseqNr = int(val["value"],16)
		imw_object_rx_list[objseqNr] = imw_object(objseqNr,globalTime)
		if(lastActHookTime > 0):
			timingCorrection = 10000000 - (globalTime - lastActHookTime)
			print(timingCorrection)
		lastActHookTime = globalTime
		
	if(hookID == "0007"):
		objseqNr = int(val["value_u16"],16)
		fragId = int(val["value_l16"],16)
		chunk = int(fragId / 10)
		currentChunk = chunk
		currentSeq = objseqNr
		try:
			obj = imw_object_rx_list[objseqNr]
			deltaTime = globalTime - obj.startTime - chunk * chunkDelay + timingCorrection
			obj.deltaTime[fragId] = deltaTime
			obj.deltaTime2[chunk] = deltaTime
		except:
			print("Can not find rx obj with id "+val["value_u16"])
			pass	
			
	if(hookID == "8030"):
		globalTs["lastDMAStart"] = globalTime
	if(hookID == "8031"):
		dmaTime = globalTime - globalTs["lastDMAStart"]

	if(hookID == "000E"):
		globalTs["lastCtxSwTime"] = globalTime
	if(hookID == "8028"):
		globalTs["lastIRQStartTime"] = globalTime
	if(hookID == "8029"):
		globalTs["lastIRQEndTime"] = globalTime
		timing["irqDurations"].append(globalTime - globalTs["lastIRQStartTime"])
	if(hookID == "8020"):
		globalTs["lastRxIRQStartTime"] = globalTime
	if(hookID == "8021"):
		globalTs["lastRxIRQEndTime"] = globalTime
		timing["rxIrqDurations"].append(globalTime - globalTs["lastRxIRQStartTime"])
	if(hookID == "8025"):
		globalTs["lastTxIRQStartTime"] = globalTime
	if(hookID == "8026"):
		globalTs["lastTxIRQEndTime"] = globalTime
		timing["txIrqDurations"].append(globalTime - globalTs["lastTxIRQStartTime"])
	if(hookID == "8000"):
		globalTs["lastRxFilterStartTime"] = globalTime
	if(hookID == "8002"):
		globalTs["lastRxFilterEndTime"] = globalTime
		delta = globalTime - globalTs["lastRxFilterStartTime"]
		timing["rxFilterResponseTime"].append(delta)
		if(globalTs["lastIRQStartTime"] < globalTs["lastRxFilterStartTime"] and \
			globalTs["lastCtxSwTime"] < globalTs["lastRxFilterStartTime"]):
			#no IRQ during filter execution
			timing["rxFilterExecutionTime"].append(delta)
	if(hookID == "8007"):
		# Store ctx switch time between MW and TX Filter task
		try:
			if(globalTs["lastSemPend"] < globalTs["lastSemPush"] and 
			globalTs["lastCtxSwTime"] > globalTs["lastSemPush"] and 
			globalTs["lastTimeTick"] < globalTs["lastSemPush"]):
				timing["ctxSwTime"].append(globalTs["lastSemDone"] - globalTs["lastSemPush"])
				#if(globalTs["lastSemDone"] - globalTs["lastSemPush"] > 1500):
					#print(lineNr)
		except KeyError:
			pass
		globalTs["lastTxFilterStartTime"] = globalTime
		
	if(hookID == "8008"):
		globalTs["lastTxFilterEndTime"] = globalTime
		delta = globalTime - globalTs["lastTxFilterStartTime"]
		timing["txFilterResponseTime"].append(delta)
		if(globalTs["lastIRQStartTime"] < globalTs["lastTxFilterStartTime"]and \
			globalTs["lastCtxSwTime"] < globalTs["lastTxFilterStartTime"]):
			#no IRQ during filter execution
			timing["txFilterExecutionTime"].append(delta)
			
	if(hookID == "0000" and testRunning == 1):
		globalTs["lastSleepTime"] = globalTime
		try:
			globalTs["executionTime"] += globalTime - globalTs["lastWakeupTime"]
		except KeyError:
			globalTs["executionTime"] = 0	
	if(hookID == "0001" and testRunning == 1):
		globalTs["lastWakeupTime"] = globalTime
		try:
			globalTs["sleepTime"] += globalTime - globalTs["lastSleepTime"]
		except KeyError:
			globalTs["sleepTime"] = 0
	
	if(hookID == "8040"):
		globalTs["lastSemPush"] = globalTime
	if(hookID == "8041"):
		globalTs["lastSemPend"] = globalTime
	if(hookID == "8042"):
		globalTs["lastSemDone"] = globalTime
		
		
				
with open(filename) as f:
	lineNr = 1
	for line in f:
		if(line.startswith("-")):
			lis = line.split()
			if(len(lis) > 3):			
				if(lineNr > 1):
					deltaTime = int('{:.9f}'.format(float(lis[3])).replace('.', ''));
					globalTime = globalTime + deltaTime
				parseLine(lis[1],lis[2], deltaTime, globalTime)
				lineNr = lineNr +1
			
	txObjMissing =0
	rxObjMissing =0
	f = open(outfolder+basename+"_rx.csv", 'w')
	for identifier in range(rxRequired):
		string=""
		try:
			obj = imw_object_rx_list[identifier]
			string = str(obj.objId)
			rxFragmentMissing = 0
			for fragment in range(rxFragments):
				try:
					time = obj.deltaTime[fragment]
					string = string + ";" + str(time)
				except:
					rxFragmentMissing += 1
					string = string + ";"
			if rxFragmentMissing != 0:
				rxObjMissing += 1
		except:
			rxObjMissing += 1
		if(string != ""):
			f.write(string + "\n")	
	f.close()
	
	f = open(outfolder+basename+"_chunk_rx.csv", 'w')
	for identifier in range(rxRequired):
		string=""
		try:
			obj = imw_object_rx_list[identifier]
			string = str(obj.objId)
			for time in obj.deltaTime2.values():
				string = string + ";" + str(time)
		except:
			pass
		if(string != ""):
			f.write(string + "\n")	
	f.close()
	
	#f = open(outfolder+basename+"_ctxsw.csv", 'w')
	#for t in timing["ctxSwTime"]:
	#	f.write(str(t)+"\n")
	#f.close()
	
	#f = open(outfolder+basename+"_timings.csv", 'w')
	#f.write("Timing;min;median;max\n")
	#for key, value in timing.items():
	#	string=key
	#	string += ";"+str(np.min(value))
	#	string += ";"+str(np.median(value))
	#	string += ";"+str(np.max(value))
	#	f.write(string + "\n")	
	#f.close()
	
	f = open(outfolder+basename+"_tx.csv", 'w')
	for identifier in range(txRequired):
		 string = ""
		 try:
			 obj = imw_object_tx_list[identifier]
			 string = str(obj.objId)
			 txFragmentMissing = 0
			 for fragment in range(txFragments):
				 try:
					 time = obj.deltaTime[fragment]
					 string = string + ";" + str(time)
				 except:
					 txFragmentMissing += 1
					 string = string + ";"
				 if txFragmentMissing != 0:
					 txObjMissing += 1
		 except:
			 txObjMissing += 1
		 if(string != ""):
			 f.write(string + "\n")
	f.close()
	
	
	#f = open(outfolder+basename+"_tx_missing.csv", 'w')
	#f.write(str(txObjMissing))
	#print("TX Missing: " + str(txObjMissing))
	#f.close()
	
	#f = open(outfolder+basename+"_rx_missing.csv", 'w')
	#f.write(str(rxObjMissing))
	#f.close()
	#print("RX Missing: " + str(rxObjMissing))
	#print("Idle Time: " + str(idle_time))
	#print("Execution Time: " + str(globalTs["executionTime"])) 
	#print("Idle Time: " + str(globalTs["sleepTime"]))   
	#idleRatio = float(globalTs["sleepTime"]) / float(globalTs["sleepTime"] + globalTs["executionTime"]) * 100
	#print("IdleRatio: " + str(idleRatio))
	
	#idle_time_ms = idle_time / 1000000
	#print("Idle Time in ms: " + str(idle_time_ms))  
	#idle_time_s = idle_time_ms / 1000
	#print("Idle Time in s: " + str(idle_time_s))
	#app_duration = globalTime - idle_check_time
	#app_duration_s = app_duration / 1000000000
	#globalTime_s = globalTime / 1000000000
	#idle_percent = idle_time_s/app_duration_s*100
	#print("Global Time in s: " + str(globalTime_s))  
	#print("Test duration in s: " + str(app_duration_s))    
	#print("Test start in s: " + str(app_start_time/1000000000)) 
	print("Done")
